#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

__version__ = '0.12.0'
version = __version__  # backwards compatibility name
